/*
 * =====================================================================================
 *
 *       Filename:  camera.cpp
 *
 *    Description:  V4L CAMERA CLASS
 *
 *        Version:  1.0
 *        Created:  2008년 08월 26일 23시 40분 57초
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Sangwoo Kim, (nanum99@korea.com) 
 *        Company:  my-company
 *
 * =====================================================================================
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <iostream>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "Camera.h"
#include "ccvt.h"

CCamera::CCamera(int dev)
{
	sprintf(m_deviceName, "/dev/video%d", dev);
	m_deviceNum = 0;
	m_imgBuffer = NULL;
	m_image = NULL;


}

CCamera::~CCamera()
{
	closeDev();
}

void CCamera::openDev()
{
	m_deviceNum = open(m_deviceName, O_RDWR);
	if( m_deviceNum <=0 )
	{
		fprintf(stderr,"open error : %s\n", m_deviceName);
		return;
	}

	this->getDeviceInfo();
}

void CCamera::closeDev()
{
	if(m_deviceNum > 0)
	{
		close(m_deviceNum);
		m_deviceNum = 0;
	}
}
/*
 * 카메라의 디바이스 정보를 획득한다.
 *
 */
void CCamera::getDeviceInfo()
{
	ioctl(m_deviceNum, VIDIOCGCAP, &m_videoCaps);
	ioctl(m_deviceNum, VIDIOCGWIN, &m_videoWin);
	ioctl(m_deviceNum, VIDIOCGPICT, &m_videoPic);
	ioctl(m_deviceNum, VIDIOCGCHAN, &m_videoChan);

	m_width = m_videoWin.width;
	m_height = m_videoWin.height;
	m_area = m_width*m_height;
}

/*
 * 카메라에 변경된 디바이스 정보를 기록한다.
 */
void CCamera::setDeviceInfo(int width, int height,
		int brightness, int colour, int hue, int contrast)
{
	m_width = width;
	m_height = height;
	m_area = m_width*m_height;

	m_videoWin.x = 0;
	m_videoWin.y = 0;
	m_videoWin.width = m_width;
	m_videoWin.height = m_height;
	m_videoWin.flags = 0;
	m_videoWin.chromakey = 0;
	//m_videoChan.flags = 0;
	//m_videoChan.type = VIDEO_TYPE_CAMERA;
	//m_videoChan.norm = VIDEO_MODE_NTSC;
	m_videoPic.palette = VIDEO_PALETTE_YUV420P;
	m_videoPic.depth = 24;
	m_videoPic.brightness = 256 * brightness;
	m_videoPic.whiteness = 256 * 128;
	m_videoPic.contrast = 256 * contrast;
	m_videoPic.hue = 256 * hue;
	m_videoPic.colour = 256 * colour;

	int error;

	error = ioctl(m_deviceNum, VIDIOCSWIN, &m_videoWin);
	if(error < 0)
	{
		fprintf(stderr,"ioctl : VIDIOSWIN\n");
		return;	
	}
	error = ioctl(m_deviceNum, VIDIOCSPICT, &m_videoPic);
	if(error < 0)
	{
		fprintf(stderr,"ioctl : VIDIOSPICT\n");
		return;	
	}
	//error = ioctl(m_deviceNum, VIDIOCSCHAN, &m_videoChan);
	//if(error < 0)
//	{
//		fprintf(stderr,"ioctl : VIDIOSCHAN\n");
//		return;	
//	}
	
}

void CCamera::capture()
{
	int len;
	if(m_deviceNum <= 0)
	{
		fprintf(stderr,"open error : %s", m_deviceName);
		return;	
	}

	len = read(m_deviceNum, m_imgBuffer, m_area*2);
	if(len <=0)
	{
		fprintf(stderr,"capture error\n");
		return;
	}

	ccvt_420p_bgr24(m_width, m_height, m_imgBuffer, m_image);			/* CCVT library */

}
/*
 * 카메라 구동
 */
void CCamera::start(int width, int height, 
				int brightness, int colour, int hue, int contrast)
{
	if(isCapturing()) 
		return;


	m_width = width;
	m_height = height;
	m_area = m_width*m_height;

	this->openDev();

    m_imgBuffer = (unsigned char *)realloc(m_imgBuffer, (size_t) (m_area/4.0 * 60));
    m_image = (unsigned char *)realloc(m_image, (size_t) (m_area * 3));
	this->setDeviceInfo(width, height, brightness, colour, hue, contrast);
}

void CCamera::stop()
{
	this->closeDev();
}

void CCamera::printDeviceInfo()
{
//	cout << "test";
}

int CCamera::getWidth()
{
	return m_width;
}

int CCamera::getHeight()
{
	return m_height;
}

unsigned char * CCamera::getBuffer()
{
	return m_imgBuffer;
}
unsigned char * CCamera::getImage()
{
	return m_image;
}

bool CCamera::isCapturing()
{
	return (m_deviceNum > 1);
}
