/*
 * =====================================================================================
 *
 *       Filename:  camera.h
 *
 *    Description:  V4L CAMERA CLASS
 *
 *        Version:  1.0
 *        Created:  2008년 08월 26일 23시 26분 29초
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Sangwoo Kim, (nanum99@korea.com) 
 *        Company:  my-company
 *
 * =====================================================================================
 */

#ifndef __CAMERA_H__
#define __CAMERA_H__
#include <linux/videodev.h>
#include <iostream>

class CCamera
{
private:
	int m_deviceNum;						/* device number */
	char m_deviceName[128];		/* device name */
	struct video_capability m_videoCaps;
	struct video_window m_videoWin;
	struct video_picture m_videoPic;
	struct video_clip m_videoClips[32];
	struct video_channel m_videoChan;
	
	unsigned char * m_imgBuffer;					/* image buffer */
	unsigned char * m_image;
	int m_width;
	int m_height;
	int m_area;
public:

	CCamera(int dev=0);
	~CCamera();
	void openDev();
	void closeDev();
	void getDeviceInfo();
	void setDeviceInfo(int width=320, int height=240, 
				int brightness=128, int colour=128, int hue=128, int contrast=128);
	void printDeviceInfo();

	void start(int width=320, int height=240, 
				int brightness=128, int colour=128, int hue=128, int contrast=128);
	void stop();
	unsigned char * getBuffer();
	unsigned char * getImage();		/* get image (RGB24) */	
	int getWidth();
	int getHeight();
	void capture();
	bool isCapturing();
};

#endif // __CAMERA_H__
