/*
 ******************************************************************************
 *
 *  file name : ImgProc.cpp
 *  description : ...
 *
 ******************************************************************************
 */

#include "ImgProc.h"
#include <qpainter.h>
#include <math.h>

#define CAN 	1
#define BOTTLE 	2
#define PT 		3


CImgProc::CImgProc()
{
	// 카메라로 부터 가져올 이미지데이타를 저장할 이미지(QImage) 초기화
	m_imgCamera.create(160,120,32);
	// m_imgCamera.load("./images/cam_can.png"); // 테스트용
	
	// 카메라 생성(동적) 및 시작
	m_camera = new CCamera(0);
	m_camera->start(160, 120);

	// make histo-array sample
	makeHistoCan();
	makeHistoBottle();
	makeHistoPT();
	
	// create histoPix
	m_pixHisto.resize( 160, 120 );
	m_pixHistoCan.resize( 160, 120 );
	m_pixHistoBottle.resize( 160, 120 );
	m_pixHistoPT.resize( 160, 120 );
	
	// draw-line in QPixmap by histo-array
	histoToPix( histoCan, m_pixHistoCan );
	histoToPix( histoBottle, m_pixHistoBottle );
	histoToPix( histoPT, m_pixHistoPT );

	// 물품별 percent를 저장할 배열을 초기화			
	percents[0] = 0;
	percents[1] = 0;
	percents[2] = 0;
	
	// 분류결과 변후 초기화
	result = -1;
}

CImgProc::~CImgProc()
{
    delete m_camera;
}

//*****************************************************************************
// 카메라 버퍼에 담겨있는 내용을 QImage로 복사해온다
//*****************************************************************************
void CImgProc::capture()
{
	/*
	*/
	unsigned char * pDst = m_imgCamera.bits();
	unsigned char * pSrc = NULL;
	int h = m_camera->getHeight();
	int w = m_camera->getWidth();

	m_camera->capture();
	pSrc = m_camera->getImage();
	for(int y=0; y < h; ++y)
	{
		for(int x=0; x < w; ++x)
		{
			pDst[y*w*4 + x*4 + 0 ] = pSrc[y* w *3+ x*3 + 0];
			pDst[y*w*4 + x*4 + 1 ] = pSrc[y* w *3+ x*3 + 1];
			pDst[y*w*4 + x*4 + 2 ] = pSrc[y* w *3+ x*3 + 2];
			pDst[y*w*4 + x*4 + 3 ] = 0;
		}
	} // end for y
}

//*****************************************************************************
// 이미지 최종적으로 히스토그램화 하여, 샘플 히스토그램과 비교 및 퍼센트 계산
//*****************************************************************************
void CImgProc::imgDist()
{
//	capture();
	m_imgCapture = m_imgCamera.copy();
	
	// 캡쳐한 이미지를, Gray->Edge->Crop->Binary->Object화
	m_imgGray = toGrayImg( m_imgCapture );
	m_imgEdge = toEdgeImg( m_imgGray );
	
	m_imgBin = toBinImg( m_imgEdge );
	cropImg( m_imgBin );
	
	m_imgBin = patchBinImg( m_imgBin );
	
	m_imgObject = toObjectImg( m_imgBin);
	m_imgObject2 = patchObjectImg( m_imgObject );
	
	makeHisto( m_imgObject2 );
	histoToPix( histo, m_pixHisto );

	percents[0] = percentObj( CAN );
	percents[1] = percentObj( BOTTLE );
	percents[2] = percentObj( PT );
	
	comparePercent();
	
	qDebug( QString("result : %1").arg(result) ) ;
}

//*****************************************************************************
// 물품별 퍼센트가 담겨있는 배열로 부터 최대값을 찾아내어
// 해당 물품에 대한 번호(상수)를 result에 저장
// - 퍼센트는 80이상으로 설정
// - 퍼센트가 80미만인 경우 분류되지 못하는 것으로 처리
//*****************************************************************************
void CImgProc::comparePercent()
{
	int max=0;
	int i=0;

	for ( i =1 ; i<3 ; i++ )
	{
		if ( percents[max] < percents[i] )
		{
			max = i;
		}
	}


	if ( 80 <= percents[max] )
	{
		result = max+1;
	}
	else
	{
		result = -1;
	}
}
//*****************************************************************************
// 결과값이 들어있는지 true / false 를 반환
//*****************************************************************************
bool CImgProc::isResult()
{
	if ( result == -1 )
	{
		return false;
	}
	else
	{
		return true;
	}
}

//*****************************************************************************
// 결과 값을 반환
//*****************************************************************************
int CImgProc::getResult()
{
	return result;
}

//*****************************************************************************
// 원본이미지로 부터 grayscale 이미지를 생성
//*****************************************************************************
QImage CImgProc::toGrayImg( QImage& imgSrc )
{
	QImage img = imgSrc;
	QRgb rgb;
	int r, g, b;
	int x, y;
	int w, h;
	
	img.detach();
	w = imgSrc.width();
	h = imgSrc.height();
	
	
	for ( y=0 ; y<h ; y++)
	{
		for ( x=0 ; x<w ; x++ )
		{
			rgb = imgSrc.pixel(x,y);
			r= g= b= ( qRed(rgb) + qGreen(rgb) + qBlue(rgb) ) /3;
			img.setPixel( x, y, qRgb(r,g,b) );
		}
	}
	
	return img;
}

//*****************************************************************************
// 원본이미지로 부터 이진화된 이미지를 생성
// 이진화된 이미지의 픽셀값은 0 또는 255
//*****************************************************************************
QImage CImgProc::toBinImg( QImage& imgSrc )
{
	QImage img = imgSrc;
	QRgb rgb;
	int r, g, b;
	int x, y;
	int w, h;
	
	img.detach();
	img = toGrayImg( img );
	
	w = imgSrc.width();
	h = imgSrc.height();
	
	for( y=0; y < h; y++ )
	{
		for( x=0; x < w; x++ )
		{
			
			rgb = imgSrc.pixel( x, y );
			r= g= b= ( ( qRed(rgb)>32 ) ? 255 : 0 );
			img.setPixel( x, y, qRgb(r,g,b) );
		}
	}

	return img;
}

//*****************************************************************************
// 이진화된 이미지를 보정
// 지정된 자표의 상하픽셀을 참조
//*****************************************************************************
QImage CImgProc::patchBinImg( QImage& imgSrc )
{
	QImage img = imgSrc;
	int r;
	int x, y;
	int w, h;
	
	int maskA[9] = {   0, 1, 0
					, 0, 0, 0
					, 0, 1, 0 };
	
	img.detach();
	img = toGrayImg( img );
	
	w = imgSrc.width();
	h = imgSrc.height();
	
	for( y=1; y < h-1; y++ )
	{
		for( x=1; x < w-1; x++ )
		{
			r = maskPixelRed( imgSrc, x, y, maskA);
			if ( 500 == r )
			{
				img.setPixel( x, y, qRgb(255,255,255) );
			}
		}
	}
	
	/*
	for( y=1; y < h-1; y++ )
	{
		for( x=1; x < w-1; x++ )
		{
			r = maskPixelRed( imgSrc, x, y, maskA);
			if ( 0 == r )
			{
				img.setPixel( x, y, qRgb(0,0,0) );
			}
		}
	}
	*/

	return img;
}

//*****************************************************************************
// 물품을 담는 그릇(아크릴)도 경계선화 대상에 포함되므로
// 적당하게 없애줌 
//*****************************************************************************
void CImgProc::cropImg( QImage& img )
{
	int x, y, w, h;
	
	w = img.width();
	h = img.height();
	
	for ( y=0; y<h ; y++)
	{
		for( x=0; x<55; x++ )
		{
			img.setPixel( x, y, qRgb(0,0,0) );
		}
	}
	
	for ( y=0; y<h ; y++)
	{
		for( x=w-55; x<w; x++ )
		{
			img.setPixel( x, y, qRgb(0,0,0) );
		}
	}
}

//*****************************************************************************
// 원본이미지로부터 경계선을 추출
// 경계선 검출 마스크로 로버츠(Roberts) 마스크를 사용
//*****************************************************************************
QImage CImgProc::toEdgeImg( QImage& imgSrc )
{
	QImage img = imgSrc;
	int r1, g1, b1;
	int r2, g2, b2;
	int x, y, w, h;
	
	
	int mask1[9]={  0, 0, 0
				  , 0, 1, 0
				  , -1, 0, 0 };
	int mask2[9]={  0, 0, 0
				  , 0, 1, 0
				  , 0, 0, -1 };
  
	
	img.detach();
	w = imgSrc.width();
	h = imgSrc.height();
	

	for( y=1; y < h-1; y++ )
	{
		for( x=1; x < w-1; x++ )
		{
			r1 = maskPixelRed( imgSrc, x, y, mask1 );
			r2 = maskPixelRed( imgSrc, x, y, mask2 ); 
			
			g1 = maskPixelGreen( imgSrc, x, y, mask1 );
			g2 = maskPixelGreen( imgSrc, x, y, mask2 );
			
			b1 = maskPixelBlue( imgSrc, x, y, mask1 );
			b2 = maskPixelBlue( imgSrc, x, y, mask2 );
			
			r1 = (int)sqrt(r1*r1 + r2*r2);
			g1 = (int)sqrt(g1*g1 + g2*g2);
			b1 = (int)sqrt(b1*b1 + b2*b2);
			
			img.setPixel( x, y, qRgb(r1,g1,b1) );
		}
	}

	
	for( x=0; x < w; x++ )
	{
		img.setPixel( x, 0, qRgb(0,0,0) );
		img.setPixel( x, 1, qRgb(0,0,0) );
		img.setPixel( x, 2, qRgb(0,0,0) );
		img.setPixel( x, h-1, qRgb(0,0,0) );
	}
	
	/*
	for( y=0; y < h; y++ )
	{
		img.setPixel( 0, y, qRgb(0,0,0) );
		img.setPixel( w-1, y, qRgb(0,0,0) );
	}
	*/
	
	
	return img;
}

//*****************************************************************************
// 원본이미지로 부터 (가로로) 오브젝트화된 이미지를 생성
// 원본이미지는 이진화된 이미지여야 함
// 좌측경계에서 우측경계까지 픽셀을 채움
//*****************************************************************************
QImage CImgProc::toObjectImg( QImage& imgSrc )
{
	QImage img = imgSrc;
	
	int x, y;
	int w, h;
	int r1;
	int temp1 = 0;
	int temp2 = 0;
	
	w = imgSrc.width();
	h = imgSrc.height();
	
	img.detach();
	
	for ( y=10; y<h-1 ; y++ )
	{
		temp1 = 0;
		temp2 = 0;
		
		for ( x=10; x<w-10 ; x++ )
		{
			r1 = qRed( imgSrc.pixel(x,y) );
			// r2 = qRed( imgSrc.pixel(x+2, y) );
			if ( 255==r1 ) // && 255==r2 )
			{
				temp1 = x;
				break;
			}
		}
		
		for ( x=w-10; x>10 ; x-- )
		{
			r1 = qRed( imgSrc.pixel(x,y) );
			// r2 = qRed( imgSrc.pixel(x-2, y) );
			
			if ( 255==r1 ) // && 255==r2 )
			{
				temp2 = x;
				break;
			}
		}

		for ( x=temp1; x<temp2; x++ )
		{
			img.setPixel( x, y, qRgb(255,255,255) );
		}
	}
	
	/*
	for ( x=10; x<w-10 ; x++ )
	{
		temp1 = 0;
		temp2 = 0;
		
		for ( y=5; y<h-5 ; y++ )
		{
			if ( 255 == qRed(img.pixel(x,y)) && 255==qRed(img.pixel(x,y-1)) )
			{
				temp1 = y;
				break;
			}
		}
		
		for ( y=h-5; y>5 ; y-- )
		{
			if ( 255 == qRed(img.pixel(x,y)) && 255==qRed(img.pixel(x,y-1)) )
			{
				temp2 = y;
				break;
			}
		}

		if ( 0!=temp1 && 0!=temp2 )
		{
			for ( y=temp1; y<temp2; y++ )
			{
				img.setPixel( x, y, qRgb(255,255,255) );
			}
		}
	}
	*/
	
	return img;
}

//*****************************************************************************
// 오브젝트화된 이미지를 보정
// 지정된 좌표의 상하픽셀을 참조
//*****************************************************************************
QImage CImgProc::patchObjectImg( QImage& imgSrc )
{
	QImage img = imgSrc;
	int x, y;
	int w, h;
	int r;
	
	int mask[9] = {   0, 1, 0
					, 0, 0, 0
					, 0, 1, 0 };
	
	w = imgSrc.width();
	h = imgSrc.height();
	img.detach();
	
	for ( y=3; y<h-3 ; y++ )
	{
		for ( x=5; x<w-5 ; x++ )
		{
		    // 상하픽셀의 값이 있으면,...
			r = maskPixelRed( imgSrc, x, y, mask );
			if (r>0) r = 255;
			img.setPixel( x, y, qRgb(r,r,r) );
		}
	}
	
	/*
	for ( y=3; y<h-3 ; y++ )
	{
		for ( x=5; x<w-5 ; x++ )
		{
		    // 상하픽셀의 값이 없으면,...
			r = maskPixelRed( imgSrc, x, y, mask );
			if (r==0) r = 0;
			img.setPixel( x, y, qRgb(r,r,r) );
		}
	}
	*/
	
	return img;
}

//*****************************************************************************
// 인근픽셀( 3x3 size )에 mask값을 참조한 값을 반환
//*****************************************************************************
int CImgProc::maskPixelRed( QImage& imgSrc, int x, int y, int mask[] )
{
	int r;

	r =   mask[0]*qRed(imgSrc.pixel(x-1,y-1))
		+ mask[1]*qRed(imgSrc.pixel(x,y-1))
		+ mask[2]*qRed(imgSrc.pixel(x+1,y-1))
		+ mask[3]*qRed(imgSrc.pixel(x-1,y))
		+ mask[4]*qRed(imgSrc.pixel(x,y))
		+ mask[5]*qRed(imgSrc.pixel(x+1,y))
		+ mask[6]*qRed(imgSrc.pixel(x-1,y+1))
		+ mask[7]*qRed(imgSrc.pixel(x,y+1))
		+ mask[8]*qRed(imgSrc.pixel(x+1,y+1));
	return r;
}
int CImgProc::maskPixelGreen( QImage& imgSrc, int x, int y, int mask[] )
{
	int g;
	g =   mask[0]*qGreen(imgSrc.pixel(x-1,y-1))
		+ mask[1]*qGreen(imgSrc.pixel(x,y-1))
		+ mask[2]*qGreen(imgSrc.pixel(x+1,y-1))
		+ mask[3]*qGreen(imgSrc.pixel(x-1,y))
		+ mask[4]*qGreen(imgSrc.pixel(x,y))
		+ mask[5]*qGreen(imgSrc.pixel(x+1,y))
		+ mask[6]*qGreen(imgSrc.pixel(x-1,y+1))
		+ mask[7]*qGreen(imgSrc.pixel(x,y+1))
		+ mask[8]*qGreen(imgSrc.pixel(x+1,y+1));
	return g;
}
int CImgProc::maskPixelBlue( QImage& imgSrc, int x, int y, int mask[] )
{
	int b;
	b =   mask[0]*qBlue(imgSrc.pixel(x-1,y-1))
		+ mask[1]*qBlue(imgSrc.pixel(x,y-1))
		+ mask[2]*qBlue(imgSrc.pixel(x+1,y-1))
		+ mask[3]*qBlue(imgSrc.pixel(x-1,y))
		+ mask[4]*qBlue(imgSrc.pixel(x,y))
		+ mask[5]*qBlue(imgSrc.pixel(x+1,y))
		+ mask[6]*qBlue(imgSrc.pixel(x-1,y+1))
		+ mask[7]*qBlue(imgSrc.pixel(x,y+1))
		+ mask[8]*qBlue(imgSrc.pixel(x+1,y+1));
	
	return b;
}

//*****************************************************************************
// 원본이미지로 부터 특정 픽셀값의 빈도수 height별로 저장
// 원본이미지는 이진화된 이미지여야 함
// histo[0]에는 가장 처음 등장하는 10px이상의 인덱스를 저장
//*****************************************************************************
void CImgProc::makeHisto( QImage& imgSrc )
{
	int x, y;
	int w, h;
	
	int n=0;
	int i=0;
	
	memset( histo, 0x00, sizeof(int)*120 );
	
	w = imgSrc.width();
	h = imgSrc.height();
	
	for ( y=1; y<h-1 ; y++ )
	{
		n=0;
		for ( x=10; x<w-10; x++ )
		{
			if ( 255 == qRed( imgSrc.pixel(x,y) ) )
			{
				n++;
			}
		}
		histo[y] = n;
		// qDebug( QString("%1").arg( histo[y] ) );
	}
	
	for ( i=3; i<h-1 ; i++ )
	{
		if ( 10 < histo[i]  )
		{
			histo[0] = i;
			break;
		}
	}
	
//	patchHisto();
}

void CImgProc::patchHisto()
{
//	int temp1[120]={0,};
//	int temp2[120]={0,};
	int i=0;
	int temp=0;
	
//	temp1[119] = histo[119];
//	temp2[0] = histo[0];
	
	for ( i=3; i<120-2 ;i++ )
	{
		temp = histo[i+1] - histo[i];
		if ( 0<temp && temp<10 && histo[i-1]!=0)
		{
			histo[i] = histo[i+1];
		}
		else
		{
			temp = histo[i+2] - histo[i];
			if ( 0<temp && temp<10 && histo[i-1]!=0)
			{
				histo[i] = histo[i+2];
			}
		}
	}
	
//	memcpy( histo, temp1, 120 );
}

//*****************************************************************************
// 히스토그램 배열의 값을 참조하여 QPixmap에 라인을 그림
//*****************************************************************************
void CImgProc::histoToPix( int arr[], QPixmap& pm )
{
	QPainter p;
	int i=0;

	pm.fill();	
	p.begin( &pm );
	p.setPen( Qt::blue );
	for ( i=0 ; i<120 ; i++ )
	{
		p.drawLine(0, i, arr[i], i);
	}
	p.end();
}

//*****************************************************************************
// 물품별 퍼센트를 반환
//*****************************************************************************
int CImgProc::percentObj( int objNum )
{
	int* pSample = NULL;
	int top=0;
	int bottom=119;

	int errorRange = 5;
	
	int countMax=0;
	int count=0;
	int avg = 0;
	
	int iObject = 0;
	int iSample = 0;
	int i=0;
	
	switch( objNum )
	{
	case CAN:
		pSample = histoCan;
		break;
	case BOTTLE:
		pSample = histoBottle;
		break;
	case PT:
		pSample = histoPT;
		break;
	}
	
	iSample = pSample[0];
	iObject = histo[0];
	
	top = (iSample>iObject) ? iSample : iObject ;
	countMax = bottom-top;
	
	for ( i=top; i<bottom; i++ )
	{
		if ( 5 > pSample[iSample] )
		{
			countMax--;
		}
		else if ( (pSample[iSample]-errorRange)<=histo[iObject]
				  && histo[iObject]<=(pSample[iSample]+errorRange) )
		{
			count++;
		}
		
		iSample += 1;
		iObject += 1;
	}
	avg = count*100 / countMax;
	
	
	// debug
	QString objName;
	switch( objNum )
	{
	case CAN:
		objName = "CAN";
		break;
	case BOTTLE:
		objName = "BOTTLE";
		break;
	case PT:
		objName = "PT";
		break;
	}

	qDebug( QString("percentObj( %1 ) : height:%1 countMax:%1 count:%1  average:%1").arg(objName).arg(bottom-top).arg(countMax).arg(count).arg(avg) );
	
	/*
	for ( i=0; i<120; i++ )
	{
		qDebug( QString("histo[%1]=%1;").arg(i).arg(histo[i]) );
	}
	*/
	
	return avg;
}

// 이하는 물품별 히스토그램 SAMPLE 값 
void CImgProc::makeHistoCan()
{
	histoCan[0]=66;
	histoCan[1]=0;
	histoCan[2]=0;
	histoCan[3]=0;
	histoCan[4]=0;
	histoCan[5]=0;
	histoCan[6]=0;
	histoCan[7]=0;
	histoCan[8]=0;
	histoCan[9]=0;
	histoCan[10]=0;
	histoCan[11]=0;
	histoCan[12]=0;
	histoCan[13]=0;
	histoCan[14]=0;
	histoCan[15]=0;
	histoCan[16]=0;
	histoCan[17]=0;
	histoCan[18]=0;
	histoCan[19]=0;
	histoCan[20]=0;
	histoCan[21]=0;
	histoCan[22]=0;
	histoCan[23]=0;
	histoCan[24]=0;
	histoCan[25]=0;
	histoCan[26]=0;
	histoCan[27]=0;
	histoCan[28]=0;
	histoCan[29]=0;
	histoCan[30]=0;
	histoCan[31]=0;
	histoCan[32]=0;
	histoCan[33]=0;
	histoCan[34]=0;
	histoCan[35]=0;
	histoCan[36]=0;
	histoCan[37]=0;
	histoCan[38]=0;
	histoCan[39]=0;
	histoCan[40]=0;
	histoCan[41]=0;
	histoCan[42]=0;
	histoCan[43]=0;
	histoCan[44]=0;
	histoCan[45]=0;
	histoCan[46]=0;
	histoCan[47]=0;
	histoCan[48]=0;
	histoCan[49]=0;
	histoCan[50]=0;
	histoCan[51]=0;
	histoCan[52]=0;
	histoCan[53]=0;
	histoCan[54]=0;
	histoCan[55]=0;
	histoCan[56]=0;
	histoCan[57]=0;
	histoCan[58]=0;
	histoCan[59]=0;
	histoCan[60]=0;
	histoCan[61]=0;
	histoCan[62]=0;
	histoCan[63]=0;
	histoCan[64]=0;
	histoCan[65]=0;
	histoCan[66]=21;
	histoCan[67]=33;
	histoCan[68]=36;
	histoCan[69]=37;
	histoCan[70]=37;
	histoCan[71]=37;
	histoCan[72]=37;
	histoCan[73]=37;
	histoCan[74]=37;
	histoCan[75]=37;
	histoCan[76]=38;
	histoCan[77]=38;
	histoCan[78]=38;
	histoCan[79]=38;
	histoCan[80]=38;
	histoCan[81]=38;
	histoCan[82]=38;
	histoCan[83]=38;
	histoCan[84]=38;
	histoCan[85]=38;
	histoCan[86]=38;
	histoCan[87]=38;
	histoCan[88]=38;
	histoCan[89]=38;
	histoCan[90]=38;
	histoCan[91]=38;
	histoCan[92]=38;
	histoCan[93]=38;
	histoCan[94]=38;
	histoCan[95]=38;
	histoCan[96]=38;
	histoCan[97]=39;
	histoCan[98]=39;
	histoCan[99]=39;
	histoCan[100]=39;
	histoCan[101]=39;
	histoCan[102]=39;
	histoCan[103]=39;
	histoCan[104]=39;
	histoCan[105]=39;
	histoCan[106]=39;
	histoCan[107]=39;
	histoCan[108]=39;
	histoCan[109]=39;
	histoCan[110]=39;
	histoCan[111]=39;
	histoCan[112]=39;
	histoCan[113]=39;
	histoCan[114]=39;
	histoCan[115]=38;
	histoCan[116]=39;
	histoCan[117]=39;
	histoCan[118]=38;
	histoCan[119]=0;
}

void CImgProc::makeHistoBottle()
{
	histoBottle[0]=29;
	histoBottle[1]=0;
	histoBottle[2]=0;
	histoBottle[3]=0;
	histoBottle[4]=0;
	histoBottle[5]=0;
	histoBottle[6]=0;
	histoBottle[7]=0;
	histoBottle[8]=0;
	histoBottle[9]=0;
	histoBottle[10]=0;
	histoBottle[11]=0;
	histoBottle[12]=0;
	histoBottle[13]=0;
	histoBottle[14]=0;
	histoBottle[15]=0;
	histoBottle[16]=0;
	histoBottle[17]=0;
	histoBottle[18]=0;
	histoBottle[19]=0;
	histoBottle[20]=0;
	histoBottle[21]=0;
	histoBottle[22]=0;
	histoBottle[23]=0;
	histoBottle[24]=0;
	histoBottle[25]=0;
	histoBottle[26]=0;
	histoBottle[27]=0;
	histoBottle[28]=0;
	histoBottle[29]=11;
	histoBottle[30]=16;
	histoBottle[31]=18;
	histoBottle[32]=18;
	histoBottle[33]=18;
	histoBottle[34]=18;
	histoBottle[35]=18;
	histoBottle[36]=18;
	histoBottle[37]=19;
	histoBottle[38]=19;
	histoBottle[39]=19;
	histoBottle[40]=18;
	histoBottle[41]=18;
	histoBottle[42]=18;
	histoBottle[43]=17;
	histoBottle[44]=17;
	histoBottle[45]=17;
	histoBottle[46]=17;
	histoBottle[47]=18;
	histoBottle[48]=18;
	histoBottle[49]=18;
	histoBottle[50]=18;
	histoBottle[51]=18;
	histoBottle[52]=18;
	histoBottle[53]=18;
	histoBottle[54]=19;
	histoBottle[55]=19;
	histoBottle[56]=20;
	histoBottle[57]=20;
	histoBottle[58]=20;
	histoBottle[59]=20;
	histoBottle[60]=20;
	histoBottle[61]=20;
	histoBottle[62]=20;
	histoBottle[63]=20;
	histoBottle[64]=20;
	histoBottle[65]=21;
	histoBottle[66]=21;
	histoBottle[67]=22;
	histoBottle[68]=22;
	histoBottle[69]=23;
	histoBottle[70]=25;
	histoBottle[71]=25;
	histoBottle[72]=27;
	histoBottle[73]=29;
	histoBottle[74]=31;
	histoBottle[75]=33;
	histoBottle[76]=34;
	histoBottle[77]=35;
	histoBottle[78]=37;
	histoBottle[79]=38;
	histoBottle[80]=39;
	histoBottle[81]=40;
	histoBottle[82]=40;
	histoBottle[83]=41;
	histoBottle[84]=41;
	histoBottle[85]=41;
	histoBottle[86]=41;
	histoBottle[87]=41;
	histoBottle[88]=41;
	histoBottle[89]=41;
	histoBottle[90]=41;
	histoBottle[91]=41;
	histoBottle[92]=41;
	histoBottle[93]=41;
	histoBottle[94]=41;
	histoBottle[95]=41;
	histoBottle[96]=41;
	histoBottle[97]=41;
	histoBottle[98]=41;
	histoBottle[99]=41;
	histoBottle[100]=41;
	histoBottle[101]=41;
	histoBottle[102]=42;
	histoBottle[103]=42;
	histoBottle[104]=42;
	histoBottle[105]=42;
	histoBottle[106]=42;
	histoBottle[107]=42;
	histoBottle[108]=42;
	histoBottle[109]=42;
	histoBottle[110]=42;
	histoBottle[111]=42;
	histoBottle[112]=43;
	histoBottle[113]=43;
	histoBottle[114]=43;
	histoBottle[115]=43;
	histoBottle[116]=43;
	histoBottle[117]=43;
	histoBottle[118]=43;
	histoBottle[119]=0;
}

void CImgProc::makeHistoPT()
{
	histoPT[0]=20;
	histoPT[1]=0;
	histoPT[2]=0;
	histoPT[3]=0;
	histoPT[4]=0;
	histoPT[5]=0;
	histoPT[6]=0;
	histoPT[7]=0;
	histoPT[8]=0;
	histoPT[9]=0;
	histoPT[10]=0;
	histoPT[11]=0;
	histoPT[12]=0;
	histoPT[13]=0;
	histoPT[14]=0;
	histoPT[15]=0;
	histoPT[16]=0;
	histoPT[17]=0;
	histoPT[18]=0;
	histoPT[19]=0;
	histoPT[20]=12;
	histoPT[21]=18;
	histoPT[22]=20;
	histoPT[23]=22;
	histoPT[24]=22;
	histoPT[25]=22;
	histoPT[26]=22;
	histoPT[27]=23;
	histoPT[28]=23;
	histoPT[29]=23;
	histoPT[30]=23;
	histoPT[31]=23;
	histoPT[32]=23;
	histoPT[33]=23;
	histoPT[34]=23;
	histoPT[35]=23;
	histoPT[36]=23;
	histoPT[37]=22;
	histoPT[38]=20;
	histoPT[39]=21;
	histoPT[40]=20;
	histoPT[41]=19;
	histoPT[42]=2;
	histoPT[43]=1;
	histoPT[44]=0;
	histoPT[45]=0;
	histoPT[46]=0;
	histoPT[47]=0;
	histoPT[48]=0;
	histoPT[49]=0;
	histoPT[50]=0;
	histoPT[51]=0;
	histoPT[52]=0;
	histoPT[53]=1;
	histoPT[54]=2;
	histoPT[55]=1;
	histoPT[56]=2;
	histoPT[57]=1;
	histoPT[58]=0;
	histoPT[59]=0;
	histoPT[60]=0;
	histoPT[61]=0;
	histoPT[62]=0;
	histoPT[63]=1;
	histoPT[64]=1;
	histoPT[65]=1;
	histoPT[66]=1;
	histoPT[67]=0;
	histoPT[68]=0;
	histoPT[69]=0;
	histoPT[70]=0;
	histoPT[71]=0;
	histoPT[72]=0;
	histoPT[73]=0;
	histoPT[74]=0;
	histoPT[75]=26;
	histoPT[76]=40;
	histoPT[77]=45;
	histoPT[78]=46;
	histoPT[79]=46;
	histoPT[80]=46;
	histoPT[81]=46;
	histoPT[82]=46;
	histoPT[83]=46;
	histoPT[84]=46;
	histoPT[85]=46;
	histoPT[86]=46;
	histoPT[87]=46;
	histoPT[88]=46;
	histoPT[89]=46;
	histoPT[90]=46;
	histoPT[91]=46;
	histoPT[92]=46;
	histoPT[93]=45;
	histoPT[94]=45;
	histoPT[95]=45;
	histoPT[96]=45;
	histoPT[97]=45;
	histoPT[98]=45;
	histoPT[99]=45;
	histoPT[100]=45;
	histoPT[101]=45;
	histoPT[102]=45;
	histoPT[103]=45;
	histoPT[104]=45;
	histoPT[105]=45;
	histoPT[106]=45;
	histoPT[107]=45;
	histoPT[108]=45;
	histoPT[109]=45;
	histoPT[110]=45;
	histoPT[111]=45;
	histoPT[112]=40;
	histoPT[113]=39;
	histoPT[114]=2;
	histoPT[115]=7;
	histoPT[116]=0;
	histoPT[117]=0;
	histoPT[118]=0;
	histoPT[119]=0;
}
