/*
 ******************************************************************************
 *
 *  file name : ImgProc.h
 *  description : ...
 *
 ******************************************************************************
 */

#ifndef CIMGPROC_H
#define CIMGPROC_H

#include <qvariant.h>
#include <qimage.h>

#include "Camera.h"

class CImgProc
{
public:
    CImgProc();
    ~CImgProc();

	// 이미지 변환 함수
    QImage toGrayImg( QImage & imgSrc );
    QImage toEdgeImg( QImage & imgSrc );
	void cropImg( QImage & img );
    QImage toBinImg( QImage & imgSrc );
    QImage patchBinImg( QImage & imgSrc );

    QImage toObjectImg( QImage & imgSrc );
    QImage patchObjectImg( QImage & imgSrc );
    
    // 이미지를 인접픽셀값을 참조하여 필터링
    int maskPixelRed( QImage & imgSrc, int x, int y, int mask[] );
    int maskPixelGreen( QImage & imgSrc, int x, int y, int mask[] );
    int maskPixelBlue( QImage & imgSrc, int x, int y, int mask[] );

	// 이미지의 특정 픽셀값의 빈도수를 histo[] 배열에 저장
    void makeHisto( QImage & imgSrc );
    void patchHisto();
    
    // CAN, BOTTLE, PT에 대한 샘플 히스토그램
	void makeHistoCan();
    void makeHistoBottle();
    void makeHistoPT();
    
    // 픽셀의 빈도수가 담겨있는 배열을 참조하여 QPixmap에 라인을 그림
	void histoToPix( int arr[], QPixmap& pm );
    
    // 오브젝트 번호에 따라 샘플 히스토그램과 비교 및 pecent 계산
    int percentObj( int objNum );
    
    // percents[]배열의 값들을 비교 및 최대값 저장
    void comparePercent();

	// 분류결과가 나왔는지
	bool isResult();
	
	// 결과값 return
    int getResult();

	// 카메라로 부터 이미지를 읽어옴
    void capture();
    
    // 이미지를 분류
    void imgDist();

    CCamera* m_camera;
    QImage m_imgCamera;
    QImage m_imgCapture;
    
    QImage m_imgGray;
    QImage m_imgEdge;
	QImage m_imgEdge2;
    QImage m_imgBin;
    QImage m_imgObject;
    QImage m_imgObject2;
    
    QPixmap m_pixHisto;
	
	// test
	QPixmap m_pixHistoCan;
	QPixmap m_pixHistoBottle;
	QPixmap m_pixHistoPT;
	
private:
    int percents[3];		// 각 물품별 percent값
    int result;			// 분류 결과값
	
    int histo[120];		// 현재 오브젝트의 히스토그램
	int histoCan[120];		// CAN(SAMPLE) 히스토그램
    int histoBottle[120];	// BOTTLE(SAMPLE) 히스토그램
    int histoPT[120];		// PT(SAMPLE) 히스토그램
};

#endif // CIMGPROC_H
