/*
 * =====================================================================================
 *
 *       Filename:  rfid.h
 *
 *    Description:  RFID READ/WRITE CLASS
 *    				ISO15693 PROTOCOL
 *
 *        Version:  1.0
 *        Created:  2008년 09월 01일 20시 46분 51초
 *       Revision:  none
 *       Compiler:  g++
 *
 *         Author:  Sangwoo Kim, nanum99@korea.com 
 *        Company:  my-company
 *
 * =====================================================================================
 */

#ifndef __RFID_H__
#define __RFID_H__

#define MAX_SN_SIZE 8
#define MAX_SN_STRING_SIZE 17
#define MAX_DATA_SIZE 128
#define BLOCK_SIZE 4
#define MAX_BLOCK_COUNT 8
#define SUCCESS 1
#define FAIL 0

#include <iostream>

class CRfid
{
private:
	unsigned char m_sn[MAX_SN_SIZE];				/* serial number */
	char m_snString[MAX_SN_STRING_SIZE];
	char m_deviceName[64];
	int m_device;
	int m_portNo;
	char m_data[MAX_DATA_SIZE];
	int hexSnToStr(unsigned char * srcStr, char * dstStr);
	unsigned short calcCRC16(unsigned short * pdata, int size);/* CRC 생성 */
	bool checkCRC16(unsigned short * pdata, int size);		/* CRC 검사 */
	void setIso(unsigned short * isoBuffer, int size);

public:
	CRfid(int dev=0);
	~CRfid();
	void rfOpen();
	void readSerial();
	void readDataBlock(unsigned short start = 0x00, unsigned short end = 0x04);
	void writeDataBlock(char * str);
	
	unsigned char * getSerialNo();	
	char * getSNString();
	char * getData();
	
	bool isSerial();	// 시리얼번호가 있는지 
	

	/*
	 * 확장
	 */
	void requestRF(unsigned short pbuffer, int size);
	void writeCommand();
	void readCommand();
};

#endif

