/*
 * =====================================================================================
 *
 *       Filename:  uart.cpp
 *
 *    Description:  RS-232
 *
 *        Version:  1.0
 *        Created:  2008년 09월 03일 20시 34분 31초
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Sangwoo Kim, nanum99@korea.com 
 *        Company:  my-company
 *
 * =====================================================================================
 */

#include "uart.h"
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <stdio.h>

CUart::CUart(char * deviceName)
{
	m_fd = 0;
	setDeviceName(deviceName);	
}


CUart::~CUart()
{
	uClose();
}

void CUart::setDeviceName(char * str)
{
	assert(str != NULL);
	strcpy(m_deviceName, str);
}

void CUart::uOpen(unsigned int option)
{
	struct termios newTerm;
	struct termios oldTerm;

	if(m_fd > 1)
	{
		fprintf(stderr, "%s is open status \n", m_deviceName);
		return;
	}

	m_fd = open(m_deviceName, O_RDWR | O_NOCTTY | O_NONBLOCK);

	if( m_fd < 1) 
	{
		fprintf(stderr, "%s open error\n", m_deviceName);
		return;
	}
	
	tcgetattr(m_fd, &oldTerm);
	memset(&newTerm, 0x00, sizeof(newTerm));

	newTerm.c_cflag = option;
	newTerm.c_cc[VTIME] = 0;
	newTerm.c_cc[VMIN] = 1;
	newTerm.c_iflag = IGNPAR | ICRNL;
	newTerm.c_oflag = 0;
	tcflush(m_fd, TCIFLUSH);

	tcsetattr(m_fd, TCSANOW, &newTerm);
}

void CUart::uClose()
{
	if(m_fd > 1)
		close(m_fd);
}

int CUart::uWrite(char * buffer, unsigned int size)
{
	return write(m_fd, buffer, size);
}

int CUart::uRead(char * buffer, unsigned int size)
{
	return read(m_fd, buffer, size);
}

