/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/

#include "hangul.h"
#include "rfid.h"
#include "uart.h"
#include "ImgProc.h"

#include <qlabel.h>

#include <stdlib.h>
#include <termios.h>
#include <unistd.h>
#include <arpa/inet.h>

#define CARDIN 0
#define CARDREAD 1
#define MENU 2
#define WAIT 3
#define DIST 4
#define GAME 5
#define RESULT 6
#define CHECKMONEY 7

#define SERVER_IP "58.227.54.159"
#define SERVER_PORT 5555

//*****************************************************************************
// 패킷 종류
//*****************************************************************************
#define CTS_REQ_USER			1001	// 사용자 확인 요청
#define STC_RES_USER_OK			1002	// 사용자 확인 성공
#define STC_RES_USER_FAIL		1003	// 사용자 확인 실패	

#define CTS_REQ_MONEY			2001	// 적립금 조회 요청
#define STC_RES_MONEY_OK		2002	// 적립금 조회 성공
#define STC_RES_MONEY_FAIL		2003	// 적립금 조회 실패	

#define CTS_REQ_REFUND			3001	// 환급금액 조회 요청
#define STC_RES_REFUND_OK		3002	// 환급금액 조회 성공
#define STC_RES_REFUND_FAIL		3003	// 환급금액 조회 실패

#define CTS_SND_RECOVERY		4001	// 회수결과 전송
#define STC_RES_RECOVERY_OK		4002	// 회수결과 전송 성공
#define STC_RES_RECOVERY_FAIL	4003	// 회수결과 전송 실패

#define CTS_SND_POINT			5001	// 적립금 전송
#define STC_RES_POINT_OK		5002	// 적립금 전송 성공	
#define STC_RES_POINT_FAIL		5003	// 적립금 전송 실패	

//*****************************************************************************
// 물품
//*****************************************************************************
#define CAN 1
#define BOTTLE 2
#define PT 3

//*****************************************************************************
// 모터 제어를 위한 상수
//*****************************************************************************
#define IMGPROC_READY		"r"		// 영상처리를 위한 준비, 카메라 위치로
#define IMGPROC_RETRY		"t"		// 영상처리 재시도, 회전
#define IMGPROC_FAIL		"f"		// 영상처리 실패, 초기 위치로

#define IMGPROC_RES_CAN		"c"		// 영상처리 결과 - 캔
#define IMGPROC_RES_BOTTLE	"b"		// 영상처리 결과 - 병
#define IMGPROC_RES_PT		"p"		// 영상처리 결과 - PT

//*****************************************************************************
// device 파일
//*****************************************************************************
#define DEV_SERIAL_RFID			0				// "/dev/ttyS0"
#define DEV_SERIAL_ATMEGA128	"/dev/ttyS2"

//*****************************************************************************
// - 생성자 에서 호출 되는 함수 -
//
// 각종 변수를 초기화
// 화면별로 구성된 위젯들에 대한 포인터를 배열에 저장
// 시그널과 슬롯을 연결
// 타이머 생성 : 카드 읽기, 영상처리
// 자식위젯들의 각종 라벨에 대한 포인터를 저장
//*****************************************************************************
void CWMain::init()
{
    m_statePrev = 0;    
    m_stateNow = 0;
    m_stateNext = 0;

    wCardIn = new CWCardIn( this );
	wCardRead = new CWCardRead( this );
	
    wMenu = new CWMenu( this );
    wWait = new CWWait( this );
	wDist = new CWDist( this );
	
	wGame = new CWGame( this );
    wResult = new CWResult( this );

    wCheckMoney = new CWCheckMoney( this );

    wCardIn->hide();
	wCardRead->hide();

    wMenu->hide();
    wWait->hide();
	wDist->hide();
	
    wResult->hide();
	wGame->hide();
    wCheckMoney->hide();
	 
	widgets[ CARDIN ] = wCardIn;
	widgets[ CARDREAD ] = wCardRead;
	
	widgets[ MENU ] = wMenu;
	
	widgets[ WAIT ] = wWait;
	widgets[ DIST ] = wDist;
	
	widgets[ GAME ] = wGame;
	widgets[ RESULT ] = wResult;
	
	widgets[ CHECKMONEY ] = wCheckMoney;

	widgets[ CARDIN ]->show();

	//-------------------
	// 시그널-슬롯 연결
	//-------------------

    // 카드 투입 화면
    connect( wCardIn, SIGNAL( signalCardInOk() ),
			 SLOT( slotCardInOk() )  );
	
	// 카드 인식 화면
	connect( wCardRead, SIGNAL( signalCardReadOk() ),
			 SLOT( slotCardReadOk() )  );
	
	connect( wCardRead, SIGNAL( signalCardReadFail() ),
			 SLOT( slotCardReadFail() )  );
    						
	// 메뉴 선택화면
	connect( wMenu,	SIGNAL( signalMenuInsert() ),
			 SLOT( slotMenuInsert() )  );
									
	connect( wMenu, SIGNAL( signalMenuCheck() ),
			 SLOT( slotMenuCheck() )  );
	
	connect( wMenu, SIGNAL( signalMenuCancel() ),
			 SLOT( slotMenuCancel() )  );	

	// 재활용품 투입 대기 화면
	connect( wWait,	SIGNAL( signalWaitOk() ),
			 SLOT( slotWaitOk() )  );

	connect( wWait,	SIGNAL( signalWaitCancel() ),
			 SLOT( slotWaitCancel() )  );
	
	// 분류 중 화면
	connect( wDist,	SIGNAL( signalDistOk() ),
			 SLOT( slotDistOk() )  );
	connect( wDist,	SIGNAL( signalDistFail() ),
			 SLOT( slotDistFail() )  );
	
	// 적립결과 화면
	connect( wResult, SIGNAL( signalResultOk() ),
			 SLOT( slotResultOk() )  );

	// 게임 화면
	connect( wGame,	SIGNAL( signalGameEnd() ),
			 SLOT( slotGameEnd() )  );
	
	// 적립금 조회 화면
	connect( wCheckMoney, SIGNAL( signalCheckMoneyOk() ),
			 SLOT( slotCheckMoneyOk() )  );


	// 상태값 변경 및 현재상태에 따른 화면 전환
	connect(	this,	SIGNAL( signalSetState(int) ),
						SLOT( slotSetState(int) ) );
						
	connect(	this,	SIGNAL( signalSwapScreen() ),
						SLOT( slotSwapScreen() ) );
	
	// 각종 변수 세팅
	m_reclaimerID = "GGK1";
	initVar();
	
	// 라벨
	labelCardID = wCardRead->getPtrLabelCardID();
	labelTotalMoney = wCheckMoney->labelTotalMoney;
	labelType = wDist->getPtrLabelType();
	labelTypeMoney = wDist->getPtrLabelMoney();
	
	// 타이머-카드읽기
	timerReadTag = new QTimer( this );
	connect( timerReadTag, SIGNAL( timeout() ), SLOT( slotReadTag() )  );
  
	timerReadTag->start( 1000, FALSE );
	
	// 타이머-분류
	timerDist = new QTimer( this );
	connect( timerDist, SIGNAL( timeout() ), SLOT( slotDist() )  );
	
	/* 안됨 
	// 타이머- sleep모드 방지를 위한 화면 repaint()
	timerRefresh = new QTimer( this );
	// connect( timerRefresh, SIGNAL( timeout() ), SLOT( repaint() )  );
	connect( timerRefresh, SIGNAL( timeout() ), SLOT( slotRefresh() )  );
	timerRefresh->start( 30*1000, FALSE );
	*/
	
	sock = socket(PF_INET, SOCK_DGRAM, 0);    
	// if(sock == -1) 
		//
	 
	memset(&serv_addr, 0, sizeof(serv_addr)); 
	serv_addr.sin_family = AF_INET; 
	serv_addr.sin_addr.s_addr = inet_addr( SERVER_IP ); 
	serv_addr.sin_port = htons( SERVER_PORT );
	
	packet_size = sizeof(m_sendPacket);
	addr_size = sizeof(serv_addr);
	
}

// 변수 초기화, m_tagID, m_cardID, m_totalMoney, m_typeMoney, m_gameMoney
void CWMain::initVar()
{
	m_tagID = "";
	m_cardID = "";
	
	m_totalMoney = 0;
	m_typeMoney = 0;
	m_gameMoney = 0;
	
	m_typeNum = 0;
}

//*****************************************************************************
// RFID 시리얼 번호를 읽어옴
//*****************************************************************************
void CWMain::slotReadTag()
{
	CRfid rfid( DEV_SERIAL_RFID );
	
	rfid.rfOpen();
	rfid.readSerial();
	
	if (1) // rfid.isSerial() )
	{
		timerReadTag->stop();
		// m_tagID = rfid.getSNString();
		m_tagID = "E004010009496387";
		slotCardInOk();
		
		// 패킷 전송
		sendPacket( CTS_REQ_USER );
		
		// 패킷 수신
		// result = (bool)recvPackcet( CTS_REQ_USER );
	
		
		if ( recvPacket( CTS_REQ_USER ) )
		{
			// 올바른 카드면
			// Card읽기 성공화면 보여줌
			labelCardID->setText( m_cardID );
			labelTotalMoney->setNum( m_totalMoney );
			QTimer::singleShot( 1*1000, wCardRead, SLOT( slotReadOk() ) );
		}
		else
		{
			// 올바른 카드가 아니면
			QTimer::singleShot( 1*1000, wCardRead, SLOT( slotReadFail() )  );
		}
	}
}

//*****************************************************************************
// 영상처리 및 재활용품 분류, 모터제어
//*****************************************************************************
void CWMain::slotDist()
{
	CUart uart1( DEV_SERIAL_ATMEGA128 ); // 모터
	CImgProc imgProc;	// 영상처리
	int nRetry = 0;
	
	uart1.uOpen(B9600 | CS8 | CLOCAL | CREAD);
	uart1.uWrite( IMGPROC_READY, 1 );
	sleep(1);

	while ( nRetry < 3 )
	{
		uart1.uWrite( IMGPROC_RETRY, 1 );
		sleep(1);
		imgProc.capture();
		imgProc.imgDist();
		
		// 영상처리 결과 확인
		if ( imgProc.isResult() )
		{
			break; // while() break
		}
		else
		{
			qDebug( " DistFail.. retry " );
			nRetry++;
		}
			
	} // while() end
	
	if ( imgProc.isResult() )
	{
		m_typeNum = imgProc.getResult();
		switch( m_typeNum )
		{
		case CAN:
			uart1.uWrite( IMGPROC_RES_CAN, 1 );
			m_typeID = "A0001";
			break;
		case BOTTLE:
			uart1.uWrite( IMGPROC_RES_BOTTLE, 1 );
			m_typeID = "B0001";
			break;
		case PT:
			uart1.uWrite( IMGPROC_RES_PT, 1 );
			m_typeID = "C0001";
			break;
		}
		
		// 패킷전송 - 해당 분류ID에 따른 적립금을 가져옴
		sendPacket( CTS_REQ_REFUND );
		recvPacket( CTS_REQ_REFUND );
		
		wDist->setTypePix( m_typeNum-1 );
		wDist->setTypeText( m_typeNum-1 );
		labelTypeMoney->setNum( m_typeMoney );
		
		sendPacket( CTS_SND_RECOVERY );
		recvPacket( CTS_SND_RECOVERY );
		
		qDebug( " DistOk.. " );
		QTimer::singleShot( 1*1000, wDist, SLOT( slotDistOk() )  );
	}
	else
	{
		// 모터를 처음상태로
		sleep(1);
		uart1.uWrite( IMGPROC_FAIL, 1 );
		qDebug( " DistFail..going to <DistFail Screen>" );
		QTimer::singleShot( 1*1000, wDist, SLOT( slotDistFail() )  );
	}
}

//*****************************************************************************
// 패킷 전송
// header값에 따라, 멤버변수를을 참조하여 m_sendPacket생성 및 전송
//*****************************************************************************
void CWMain::sendPacket( int packetHeader )
{
	void* pData = m_sendPacket.data;
	
	memset( &m_sendPacket, 0x00, sizeof(m_sendPacket) );
	m_sendPacket.header = packetHeader;
	
	switch( packetHeader )
	{
	case CTS_REQ_USER :
		strcpy( ((packetData_t_CTS_REQ_USER*)pData)->tagID , m_tagID.ascii() );
		break;
		
	case CTS_REQ_MONEY :
		strcpy( ((packetData_t_CTS_REQ_MONEY*)pData)->cardID , m_cardID.ascii() );
		break;
		
	case CTS_REQ_REFUND :
		strcpy( ((packetData_t_CTS_REQ_REFUND*)pData)->productID , m_typeID.ascii() );
		break;
		
	case CTS_SND_RECOVERY :
		strcpy( ((packetData_t_CTS_SND_RECOVERY*)pData)->productID , m_typeID.ascii() );
		strcpy( ((packetData_t_CTS_SND_RECOVERY*)pData)->cardID , m_cardID.ascii() );
		strcpy( ((packetData_t_CTS_SND_RECOVERY*)pData)->reclaimerID , m_reclaimerID.ascii() );
		break;
		
	case CTS_SND_POINT :
		strcpy( ((packetData_t_CTS_SND_POINT*)pData)->cardID , m_cardID.ascii() );
		((packetData_t_CTS_SND_POINT*)pData)->productMoney = m_typeMoney;
		((packetData_t_CTS_SND_POINT*)pData)->gameMoney = m_gameMoney;
		break;
	}
	
	sendto(sock, &m_sendPacket, packet_size, 0, (struct sockaddr*)&serv_addr, sizeof(serv_addr) );
	
}

//*****************************************************************************
// 패킷 수신
// m_recvPacket에 담음
// 이전에 보낸 header값에 따라 그에 대응되는 패킷을 OK header와 FAIL header를 구분
//*****************************************************************************
bool CWMain::recvPacket( int packetHeader )
{
	void* pData = m_recvPacket.data;
	bool result=false;
	
	memset( &m_recvPacket, 0x00, sizeof(m_recvPacket) );
	
	recvfrom(sock, &m_recvPacket, packet_size, 0, (struct sockaddr*)&from_addr, (socklen_t*)&addr_size);
	
	switch( packetHeader )
	{
	case CTS_REQ_USER :
		if ( m_recvPacket.header == STC_RES_USER_OK )
		{
			m_cardID.setAscii( ((packetData_t_STC_RES_USER_OK*)pData)->cardID );
			m_totalMoney = ((packetData_t_STC_RES_USER_OK*)pData)->totalMoney;
			
			result = true;
		}
		else
		{
			result = false;
		}
		
		break;
		
	case CTS_REQ_MONEY :
		if ( m_recvPacket.header == STC_RES_MONEY_OK )
		{
			m_totalMoney = ((packetData_t_STC_RES_MONEY_OK*)pData)->totalMoney;
			result = true;
		}
		else
		{
			result = false;
		}
		break;
		
	case CTS_REQ_REFUND :
		if ( m_recvPacket.header == STC_RES_REFUND_OK )
		{
			m_typeMoney = ((packetData_t_STC_RES_REFUND_OK*)pData)->productMoney;
			result = true;
		}
		else
		{
			result = false;
		}
		break;
		
	case CTS_SND_RECOVERY :
		if ( m_recvPacket.header == STC_RES_RECOVERY_OK )
		{
			result = true;
		}
		else
		{
			result = false;
		}
		break;
		
	case CTS_SND_POINT :
		if ( m_recvPacket.header == STC_RES_POINT_OK )
		{
			result = true;
		}
		else
		{
			result = false;
		}
		break;
	
	}
		
	return result;
}



//=============================================================================
// 사용자 정의 슬롯
// -> 사용자 정의 시그널 발생
// signalSetState(int) -> setState(int)
//=============================================================================
// 카드 투입화면 - 카드가 들어왔을 때
void CWMain::slotCardInOk(void)
{
	emit signalSetState( CARDREAD );
//	timerReadTag->start( 1000, FALSE );
}

// 카드 인식화면 - 카드읽기 성공시 
void CWMain::slotCardReadOk(void)
{
	emit signalSetState( MENU );
}
// 카드 인식실패화면 - 확인 버튼 클릭시
void CWMain::slotCardReadFail(void)
{
	initVar();
	timerReadTag->start( 1000, FALSE );
	emit signalSetState( CARDIN );
}

// 메뉴 선택화면 - 투입버튼 클릭시
void CWMain::slotMenuInsert(void)
{
	emit signalSetState( WAIT );
//	widgets[ WAIT ]->startTimer( 1000 ); // 1000ms
}
// 메뉴 선택화면 - 조회버튼 클릭시
void CWMain::slotMenuCheck(void)
{
	emit signalSetState( CHECKMONEY );
}

// 메뉴 선택화면 - 종료버튼 클릭시
void CWMain::slotMenuCancel(void)
{
	initVar(); 
	emit signalSetState( CARDIN );
	timerReadTag->start( 1000, FALSE );
}

// 재활용품 투입대기 화면 - 투입되었을 때
void CWMain::slotWaitOk(void)
{
	emit signalSetState( DIST );
	timerDist->start( 1000, TRUE ); // timer will be started as single shot
}

// 재활용품 투입대기 화면 - 취소 및 타임아웃
void CWMain::slotWaitCancel(void)
{
	emit signalSetState( MENU );
}

// 분류 중 화면
// Ok
void CWMain::slotDistOk(void)
{
	emit signalSetState( GAME );
}
// Fail
void CWMain::slotDistFail(void)
{
	emit signalSetState( MENU );
}

// 게임화면 종료시 (자동종료시)
void CWMain::slotGameEnd(void)
{
	m_gameMoney = wGame->money;
	m_totalMoney += m_typeMoney + m_gameMoney;
	
	labelTotalMoney->setNum( m_totalMoney );
	
	wResult->setLabels( m_typeMoney,
					   m_gameMoney,
					   m_totalMoney
					 );
	// 패킷 전송
	sendPacket( CTS_SND_POINT );
	recvPacket( CTS_SND_POINT );
	emit signalSetState( RESULT );
}

// 최종 적립결과 화면 - 확인
void CWMain::slotResultOk(void)
{
	emit signalSetState( MENU );
}
// 적립금 조회 화면 - 화인
void CWMain::slotCheckMoneyOk(void)
{
	emit signalSetState( MENU );
}



//=============================================================================
// state변경 및 state에 따른 화면 변경
//=============================================================================
void CWMain::slotSetState(int state)
{
	m_statePrev = m_stateNow;
	m_stateNow = state;
	emit signalSwapScreen();		// m_state 변경에 따른 시그널 발생
}

//-----------------------------------------------------------------------------
void CWMain::slotSwapScreen(void)
{
	widgets[ m_statePrev ]->hide();
	widgets[ m_stateNow  ]->show();
//	widgets[ m_stateNow  ] ->raise();
}

/*
void CWMain::slotRefresh(void)
{
	qDebug(" repaint()...");
	widgets[ m_stateNow ]->repaint(TRUE);
}
*/

