/*
 * gclient.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>

#define CTS_REQ_USER			1001	// 사용자 확인 요청
#define STC_RES_USER_OK			1002	// 사용자 확인 성공
#define STC_RES_USER_FAIL		1003	// 사용자 확인 실패	

#define CTS_REQ_MONEY			2001	// 적립금 조회 요청
#define STC_RES_MONEY_OK		2002	// 적립금 조회 성공
#define STC_RES_MONEY_FAIL		2003	// 적립금 조회 실패	

#define CTS_REQ_REFUND			3001	// 환급금액 조회 요청
#define STC_RES_REFUND_OK		3002	// 환급금액 조회 성공
#define STC_RES_REFUND_FAIL		3003	// 환급금액 조회 실패

#define CTS_SND_RECOVERY		4001	// 회수결과 전송
#define STC_RES_RECOVERY_OK		4002	// 회수결과 전송 성공
#define STC_RES_RECOVERY_FAIL	4003	// 회수결과 전송 실패

#define CTS_SND_POINT			5001	// 적립금 전송
#define STC_RES_POINT_OK		5002	// 적립금 전송 성공	
#define STC_RES_POINT_FAIL		5003	// 적립금 전송 실패	



#define BUFSIZE 32
#define DATA_SIZE 28

#define TAGID_SIZE 17
#define CARDID_SIZE 10
#define RECLAIMERID_SIZE 5
#define PRODUCTID_SIZE 6

void error_handling(char *message);

//*****************************************************************************
// 패킷 정의
//*****************************************************************************
struct packet_t
{
	int header;
	char data[ DATA_SIZE ];
};

//=============================================================================
// 사용자 확인 데이타
//=============================================================================
struct packetData_t_CTS_REQ_USER
{
	char tagID[ TAGID_SIZE ];
};
//----------------------------------------
struct packetData_t_STC_RES_USER_OK
{
	char cardID[ CARDID_SIZE ];
	int totalMoney;	
};
//----------------------------------------
struct packetData_t_STC_RES_USER_FAIL
{
	
};

//=============================================================================
// 총적립금 조회 데이타
//=============================================================================
struct packetData_t_CTS_REQ_MONEY
{
	char cardID[ CARDID_SIZE ];
};
//----------------------------------------
struct packetData_t_STC_RES_MONEY_OK
{
	int totalMoney;
};
//----------------------------------------
struct packetData_t_STC_RES_MONEY_FAIL
{
	
};

//=============================================================================
// 환급금액 조회 데이타
//=============================================================================
struct packetData_t_CTS_REQ_REFUND
{
	char productID[ PRODUCTID_SIZE ];
};
//----------------------------------------
struct packetData_t_STC_RES_REFUND_OK
{
	int productMoney;
};
//----------------------------------------
struct packetData_t_STC_RES_REFUND_FAIL
{
	
};

//=============================================================================
// 회수결과 전송 데이타
//=============================================================================
struct packetData_t_CTS_SND_RECOVERY
{
	char productID[ PRODUCTID_SIZE ];
	char cardID[ CARDID_SIZE ];
	char reclaimerID[ RECLAIMERID_SIZE ];
};
//----------------------------------------
struct packetData_t_STC_SND_RECOVERY_OK
{
};
//----------------------------------------
struct packetData_t_STC_SND_RECOVERY_FAIL
{
};

//=============================================================================
// 적립금 전송 데이타
//=============================================================================
struct packetData_t_CTS_SND_POINT
{
	char cardID[ CARDID_SIZE ];
	int productMoney;
	int gameMoney;
};
//----------------------------------------
struct packetData_t_STC_RES_POINT_OK
{
};
//----------------------------------------
struct packetData_t_STC_RES_POINT_FAIL
{
};

//*****************************************************************************
// main()
//*****************************************************************************
int main(int argc, char **argv)
{
	int sock;
	char message[BUFSIZE];
	int str_len, addr_size;
	
	struct sockaddr_in serv_addr;
	struct sockaddr_in from_addr;
	
	struct packet_t sendPacket, recvPacket;
	struct packetData_t_CTS_REQ_REFUND sendPacketData;
	struct packetData_t_STC_RES_REFUND_OK recvPacketData;
	
	int packet_size = sizeof(sendPacket);
	int sendPacketData_size = sizeof(sendPacketData);
	int recvPacketData_size = sizeof(recvPacketData);
	
	if(argc!=3){
		printf("Usage : %s <IP> <port>\n", argv[0]);
		exit(1);
	}
	
	sock=socket(PF_INET, SOCK_DGRAM, 0);   
	if(sock == -1)
		error_handling("socket() error");
	
	memset(&serv_addr, 0, sizeof(serv_addr));
	serv_addr.sin_family=AF_INET;
	serv_addr.sin_addr.s_addr=inet_addr(argv[1]);
	serv_addr.sin_port=htons(atoi(argv[2]));
	
	while(1)
	{
		fputs("전송할 메시지를 입력 하세요 (q to quit) : ", stdout);
		fgets(message, sizeof(message), stdin);     
		if(!strcmp(message,"q\n"))	break;
		
		memset( &sendPacket, 0x00, packet_size );
		memset( &sendPacketData, 0x00, sendPacketData_size );
		
		memset( &recvPacket, 0x00, packet_size );
		memset( &recvPacketData, 0x00, sendPacketData_size );
		
		strcpy(sendPacketData.productID, "C0001");
//		strcpy(packetData.cardID, "081001123\0");
//		strcpy(packetData.reclaimerID, "GK12\0");
		
		sendPacket.header = CTS_REQ_REFUND;
		memcpy( sendPacket.data, &sendPacketData, sendPacketData_size );
		
		
		sendto(sock, &sendPacket, packet_size, 0, (struct sockaddr*)&serv_addr, sizeof(serv_addr) );
		
		// sendto(sock, message, strlen(message), 0, (struct sockaddr*)&serv_addr, sizeof(serv_addr));
		addr_size=sizeof(from_addr);
		
		recvfrom(sock, &recvPacket, packet_size, 0, (struct sockaddr*)&from_addr, &addr_size);
		
		printf("sendPacket.header : %d\n", sendPacket.header);
		printf("productID : %s\n", ((struct packetData_t_CTS_REQ_REFUND*)(sendPacket.data))->productID );
		
		printf("recvPacket.header : %d\n", recvPacket.header);

		if ( recvPacket.header == STC_RES_REFUND_OK )
		{
			memcpy( &recvPacketData, recvPacket.data, recvPacketData_size );
			printf("PRODUCT_ID     : %s\n", sendPacketData.productID );
			printf("PRODUCT_MONEY : %d\n", recvPacketData.productMoney );
		}
		else
		{
			printf("분류ID 확인 실패...\n");
		}
	}
	
	close(sock);
	return 0;
}

void error_handling(char *message)
{
	fputs(message, stderr);
	fputc('\n', stderr);
	exit(1);
}
