/*
 * gserver.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>

#include <mysql/mysql.h>

#define DB_IP "58.227.54.159"
#define DB_NAME "grbs"
#define DB_ID "aaaa"
#define DB_PW "1111"

#define FAIL				0
#define SUCCESS				1
#define DB_CONNECTION		1
#define DB_DISCONNECTION	0

#define CTS_REQ_USER			1001	// 사용자 확인 요청
#define STC_RES_USER_OK			1002	// 사용자 확인 성공
#define STC_RES_USER_FAIL		1003	// 사용자 확인 실패	

#define CTS_REQ_MONEY			2001	// 적립금 조회 요청
#define STC_RES_MONEY_OK		2002	// 적립금 조회 성공
#define STC_RES_MONEY_FAIL		2003	// 적립금 조회 실패	

#define CTS_REQ_REFUND			3001	// 환급금액 조회 요청
#define STC_RES_REFUND_OK		3002	// 환급금액 조회 성공
#define STC_RES_REFUND_FAIL		3003	// 환급금액 조회 실패

#define CTS_SND_RECOVERY		4001	// 회수결과 전송
#define STC_RES_RECOVERY_OK		4002	// 회수결과 전송 성공
#define STC_RES_RECOVERY_FAIL	4003	// 회수결과 전송 실패

#define CTS_SND_POINT			5001	// 적립금 전송
#define STC_RES_POINT_OK		5002	// 적립금 전송 성공	
#define STC_RES_POINT_FAIL		5003	// 적립금 전송 실패	



#define BUF_SIZE 32
// #define PACKET_SIZE 32
#define DATA_SIZE 28
#define QUERY_SIZE 128

#define TAGID_SIZE 17
#define CARDID_SIZE 10
#define RECLAIMERID_SIZE 5
#define PRODUCTID_SIZE 6

//*****************************************************************************
// Global Variable
//*****************************************************************************
MYSQL gDBConnection;
int gDBConnected = 0;

//*****************************************************************************
// 패킷 정의
//*****************************************************************************
typedef struct packet_t packet_t;

struct packet_t
{
	int header;
	char data[ DATA_SIZE ];
};

//=============================================================================
// 사용자 확인 데이타
//=============================================================================
struct packetData_t_CTS_REQ_USER
{
	char tagID[ TAGID_SIZE ] ;
};
//----------------------------------------
struct packetData_t_STC_RES_USER_OK
{
	char cardID[ CARDID_SIZE ];
	int totalMoney;	
};
//----------------------------------------
struct packetData_t_STC_RES_USER_FAIL
{
	
};

//=============================================================================
// 총적립금 조회 데이타
//=============================================================================
struct packetData_t_CTS_REQ_MONEY
{
	char cardID[ CARDID_SIZE ];
};
//----------------------------------------
struct packetData_t_STC_RES_MONEY_OK
{
	int totalMoney;
};
//----------------------------------------
struct packetData_t_STC_RES_MONEY_FAIL
{
	
};

//=============================================================================
// 환급금액 조회 데이타
//=============================================================================
struct packetData_t_CTS_REQ_REFUND
{
	char productID[ PRODUCTID_SIZE ];
};
//----------------------------------------
struct packetData_t_STC_RES_REFUND_OK
{
	int productMoney;
};
//----------------------------------------
struct packetData_t_STC_RES_REFUND_FAIL
{
	
};

//=============================================================================
// 회수결과 전송 데이타
//=============================================================================
struct packetData_t_CTS_SND_RECOVERY
{
	char productID[ PRODUCTID_SIZE ];
	char cardID[ CARDID_SIZE ];
	char reclaimerID[ RECLAIMERID_SIZE ];
};
//----------------------------------------
struct packetData_t_STC_RES_RECOVERY_OK
{
};
//----------------------------------------
struct packetData_t_STC_RES_RECOVERY_FAIL
{
};

//=============================================================================
// 적립금 전송 데이타
//=============================================================================
struct packetData_t_CTS_SND_POINT
{
	char cardID[ CARDID_SIZE ];
	int productMoney;
	int gameMoney;
};
//----------------------------------------
struct packetData_t_STC_RES_POINT_OK
{
};
//----------------------------------------
struct packetData_t_STC_RES_POINT_FAIL
{
};

//*****************************************************************************
// function prototype
//*****************************************************************************
void makePacket( int packetHeader, struct packet_t * packet );
void makeQuery( struct packet_t * packet, char* strSQL );
int dbOpen( char* pIPAddr, char* pUsername, char* pPasswd, char* pDBName );
void dbClose( void );
void error_handling(char *message);

//*****************************************************************************
// main()
//*****************************************************************************
int main(int argc, char **argv)
{
	// 소켓 관련 변수
	int serv_sock;
	struct sockaddr_in serv_addr;
	struct sockaddr_in clnt_addr;
	int clnt_addr_size = sizeof(clnt_addr);;
	
	// 패킷 변수
	struct packet_t recvPacket, sendPacket;
	int packet_size = sizeof(recvPacket);
	int packet_len;
	
	// DB 관련 변수
	int result=0;
	char strSQL[ QUERY_SIZE ]={0,}; // SQL문을 담을 변수

	// 아귀먼트 검사	
	if( argc!=2 )
	{
		printf( "Usage : %s <port>\n", argv[0] );
		exit(1);
	}
	
	// UDP 소켓 생성
	serv_sock=socket(PF_INET, SOCK_DGRAM, 0);
	if( serv_sock == -1 )
	{
		error_handling("UDP 소켓 생성 오류");
	}
	
	//---------
	memset( &serv_addr, 0, sizeof(serv_addr) );
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_addr.s_addr = htonl( INADDR_ANY );
	serv_addr.sin_port = htons( atoi( argv[1] ) );
	
	// bind()
	if( bind(serv_sock, (struct sockaddr*) &serv_addr, sizeof(serv_addr)) == -1 )
	{
		error_handling("bind() error");
	}
	
	// DB 접속
	dbOpen( DB_IP, DB_ID, DB_PW, DB_NAME );
	
	while( 1 )
	{
		// 변수 초기화
		memset( &recvPacket, 0x00, packet_size );
		memset( &sendPacket, 0x00, packet_size );
		memset( strSQL, 0x00, QUERY_SIZE );
		
		// 패킷 수신
		packet_len = recvfrom(serv_sock, &recvPacket, packet_size, 0, (struct sockaddr*)&clnt_addr, &clnt_addr_size);
		
		// 쿼리 생성
		/*
		packetType = recvPacket.header;
		packetData = recvPacket.data;
		makeQuery( packetType, packetData, strSQL );
		*/
		makeQuery( &recvPacket, strSQL );
		printf("%s\n", strSQL);
		
		// DB에 질의
		result = mysql_query(&gDBConnection, strSQL);
		
		if ( result==0 )
		{	// 쿼리가 성공하면,
			makePacket( recvPacket.header, &sendPacket );
		}
		else
		{
			// 쿼리가 실패면
			sendPacket.header = recvPacket.header + 2;

		}
		
		// 패킷 전송
		sendto(serv_sock, &sendPacket, packet_size, 0, (struct sockaddr*)&clnt_addr, sizeof(clnt_addr));
	}
	
	close( serv_sock );
	return 0;
} // main() END

//*****************************************************************************
// (들어온 패킷별로) DB에 질의한 결과를 패킷화한다.
//*****************************************************************************
void makePacket( int packetHeader, struct packet_t * packet )
{
	MYSQL_RES* res_ptr;
	MYSQL_ROW sqlRow;
	void* pData = packet->data;

	switch( packetHeader )
	{
	// 사용자 확인
	case CTS_REQ_USER : 
		res_ptr = mysql_store_result(&gDBConnection);
		if( res_ptr )
		{
			if ( 1 == mysql_num_rows( res_ptr ) )
			{
				// 결과 레코드가 하나면, 성공
				packet->header = STC_RES_USER_OK;
				sqlRow = mysql_fetch_row( res_ptr );
				strncpy( ((struct packetData_t_STC_RES_USER_OK*)pData)->cardID, sqlRow[0], CARDID_SIZE );
				((struct packetData_t_STC_RES_USER_OK*)pData)->totalMoney = atoi( sqlRow[1] );
			}
			else
			{
				// 결과 레코드가 하나가 아니면, 실패
				packet->header = STC_RES_USER_FAIL;
			}
			mysql_free_result(res_ptr);
		}
		else
		{
			// 결과가 없으면, 실패
			packet->header = STC_RES_USER_FAIL;
		}
		break;
		
	// 총적립금 조회
	case CTS_REQ_MONEY :
		res_ptr = mysql_store_result(&gDBConnection);
		if( res_ptr )
		{
			if ( 1 == mysql_num_rows( res_ptr ) )
			{
				// 결과 레코드가 하나면, 성공
				packet->header = STC_RES_MONEY_OK;
				sqlRow = mysql_fetch_row( res_ptr );
				((struct packetData_t_STC_RES_MONEY_OK*)pData)->totalMoney = atoi( sqlRow[0] );
			}
			else
			{
				// 결과 레코드가 하나가 아니면, 실패
				packet->header = STC_RES_MONEY_FAIL;
			}
			mysql_free_result(res_ptr);
		}
		else
		{
			// 결과가 없으면, 실패
			packet->header = STC_RES_MONEY_FAIL;
		}
		break;
	
	// 환급금액 확인
	case CTS_REQ_REFUND :
		res_ptr = mysql_store_result(&gDBConnection);
		if( res_ptr )
		{
			if ( 1 == mysql_num_rows( res_ptr ) )
			{
				// 결과 레코드가 하나면, 성공
				packet->header = STC_RES_REFUND_OK;
				sqlRow = mysql_fetch_row( res_ptr );
				((struct packetData_t_STC_RES_REFUND_OK*)pData)->productMoney = atoi( sqlRow[0] );
			}
			else
			{
				// 결과 레코드가 하나가 아니면, 실패
				packet->header = STC_RES_REFUND_FAIL;
			}
			mysql_free_result(res_ptr);
		}
		else
		{
			// 결과가 없으면, 실패
			packet->header = STC_RES_REFUND_FAIL;
		}
		
		break;
	
	// 회수결과 전송
	case CTS_SND_RECOVERY :
		packet->header = STC_RES_RECOVERY_OK;
		break;
	
	// 적립금 전송
	case CTS_SND_POINT :
		packet->header = STC_RES_RECOVERY_OK;
		break;		
	}
} // makePacket() END

//*****************************************************************************
// 들어온 패킷을 프로토콜에 맞추어 DB에 질의할 쿼리문을 생성한다.
//*****************************************************************************
void makeQuery( struct packet_t * packet, char* strSQL )
{
	int header = packet->header;
	void* pData = packet->data;

	switch( header )
	{
	// 사용자 확인
	case CTS_REQ_USER : 
		sprintf( strSQL
				, "select CARD_ID, TOTAL_POINT from CARD_INFO where TAG_ID=\'%s\'"
				, ((struct packetData_t_CTS_REQ_USER*)pData)->tagID
				);
		break;
		
	// 총적립금 조회
	case CTS_REQ_MONEY :
		sprintf( strSQL
				, "select TOTAL_POINT from CARD_INFO where CARD_ID=\'%s\'"
				, ((struct packetData_t_CTS_REQ_MONEY*)pData)->cardID
				);
		break;
	
	// 환급금액 확인
	case CTS_REQ_REFUND :
		sprintf( strSQL
				, "select POINT from PRODUCT_INFO where PRODUCT_ID=\'%s\'"
				, ((struct packetData_t_CTS_REQ_REFUND*)pData)->productID
				);
		break;
	
	// 회수결과 전송
	case CTS_SND_RECOVERY :
		sprintf( strSQL
				, "insert into RECOVERY_INFO(PRODUCT_ID, CARD_ID, RECLAIMER_ID) values(\'%s\', \'%s\', \'%s\')"
				, ((struct packetData_t_CTS_SND_RECOVERY*)pData)->productID
				, ((struct packetData_t_CTS_SND_RECOVERY*)pData)->cardID
				, ((struct packetData_t_CTS_SND_RECOVERY*)pData)->reclaimerID
				);
		break;
	
	// 적립금 전송
	case CTS_SND_POINT :
		sprintf( strSQL
				, "insert into POINT_HISTORY(CARD_ID, POINT, ADD_POINT) values(\'%s\', %d, %d)"
				, ((struct packetData_t_CTS_SND_POINT*)pData)->cardID
				, ((struct packetData_t_CTS_SND_POINT*)pData)->productMoney
				, ((struct packetData_t_CTS_SND_POINT*)pData)->gameMoney
				);
		break;
	}
} // makeQuery() END

//*****************************************************************************
// DB 열기
//*****************************************************************************
int dbOpen( char* pIPAddr, char* pUsername, char* pPasswd, char* pDBName )
{
	if(gDBConnected) 
		return SUCCESS;

	mysql_init(&gDBConnection);
	if(!mysql_real_connect(&gDBConnection, pIPAddr, pUsername, pPasswd, pDBName,
							0,NULL,0))
	{
		fprintf(stderr,"[%s:%s] connection failure: %d, %s\n", 
				pIPAddr, pDBName,
				mysql_errno(&gDBConnection), mysql_error(&gDBConnection));

		return FAIL;
	}
	
	mysql_query(&gDBConnection, "SET NAMES UTF8");		/* CHARACTER SET UTF8 */
	gDBConnected = DB_CONNECTION;
	return SUCCESS;
}

//*****************************************************************************
// DB 닫기
//*****************************************************************************
void dbClose(void)
{
	if(gDBConnected)
	{
		mysql_close(&gDBConnection);
	}		

	gDBConnected = DB_DISCONNECTION;
}

//*****************************************************************************
// 에러메세지 출력함수
//*****************************************************************************
void error_handling(char *message)
{
	fputs(message, stderr);
	fputc('\n', stderr);
	exit(1);
}
